/*!
 * 
 * Portamento  v2.0.0b - private beta - may be buggy!
 * http://simianstudios.com/portamento
 *   
 * Copyright 2011 Kris Noble except where noted.
 * 
 * Dual-licensed under the GPLv3 and Apache 2.0 licenses: 
 * http://www.gnu.org/licenses/gpl-3.0.html
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/**
 * 
 * Creates a sliding panel that respects the boundaries of
 * a given wrapper, and also has sensible behaviour if the
 * viewport is too small to display the whole panel.
 * 
 * Full documentation at http://simianstudios.com/portamento
 * 
 * ----
 * 
 * Uses the viewportOffset plugin by Ben Alman aka Cowboy:
 * http://benalman.com/projects/jquery-misc-plugins/#viewportoffset
 * 
 * Uses a portion of CFT by Juriy Zaytsev aka Kangax:
 * http://kangax.github.com/cft/#IS_POSITION_FIXED_SUPPORTED
 * 
 * Uses code by Matthew Eernisse:
 * http://www.fleegix.org/articles/2006-05-30-getting-the-scrollbar-width-in-pixels
 * 
 * Uses the matchMedia polyfill by Scott Jehl, Paul Irish & Nicholas Zakas:
 * https://github.com/paulirish/matchMedia.js/
 * 
 * Builds on work by Remy Sharp:
 * http://jqueryfordesigners.com/fixed-floating-elements/
 *
 * ----
 *
 * TODO:
 * 1. Add multiple element support (Ant Eksiler comment)
 * 2. Allow users to overrride the viewport size checking if desired (James Eagle comment) 
 */
(function($){
  	
	$.fn.portamento = function(options) {
		
		// we'll use the window and document objects a lot, so
		// saving them as variables now saves a lot of function calls
		var thisWindow = $(window);
		var thisDocument = $(document);
		
		// set up some other vars for later use
		var panelOffset, panelMargin, realPanelOffset, topScrollBoundary;
						
		/**
		 * NOTE: included here so as to avoid namespace clashes.
		 * 
		 * jQuery viewportOffset - v0.3 - 2/3/2010
		 * http://benalman.com/projects/jquery-misc-plugins/
		 * 
		 * Copyright (c) 2010 "Cowboy" Ben Alman
		 * Dual licensed under the MIT and GPL licenses.
		 * http://benalman.com/about/license/
		 */	
	  	$.fn.viewportOffset=function(){var a=$(window);var b=$(this).offset();return{left:b.left-a.scrollLeft(),top:b.top-a.scrollTop()}}
		
		/**
		 * 
		 * A test to see if position:fixed is supported.
		 * Taken from CFT by Kangax - http://kangax.github.com/cft/#IS_POSITION_FIXED_SUPPORTED
		 * Included here so as to avoid namespace clashes.
		 * 
		 */
		function positionFixedSupported(){var a=document.body;if(document.createElement&&a&&a.appendChild&&a.removeChild){var b=document.createElement("div");if(!b.getBoundingClientRect){return null}b.innerHTML="x";b.style.cssText="position:fixed;top:100px;";a.appendChild(b);var c=a.style.height,d=a.scrollTop;a.style.height="3000px";a.scrollTop=500;var e=b.getBoundingClientRect().top;a.style.height=c;var f=e===100;a.removeChild(b);a.scrollTop=d;return f}return null}
		
		/**
		 * 
		 * Get the scrollbar width by Matthew Eernisse.
		 * http://www.fleegix.org/articles/2006-05-30-getting-the-scrollbar-width-in-pixels
		 * Included here so as to avoid namespace clashes.
		 * 
		 */
		function getScrollerWidth(){var a=null;var b=null;var c=0;var d=0;a=document.createElement("div");a.style.position="absolute";a.style.top="-1000px";a.style.left="-1000px";a.style.width="100px";a.style.height="50px";a.style.overflow="hidden";b=document.createElement("div");b.style.width="100%";b.style.height="200px";a.appendChild(b);document.body.appendChild(a);c=b.offsetWidth;a.style.overflow="auto";d=b.offsetWidth;document.body.removeChild(document.body.lastChild);return c-d}
		
		// ---------------------------------------------------------------------------------------------------
		
	   /*
	    * NOTE: included here so as to avoid namespace clashes.
	    * 
		* matchMedia() polyfill - test whether a CSS media type or media query applies
		* authors: Scott Jehl, Paul Irish, Nicholas Zakas
		* Copyright (c) 2011 Scott, Paul and Nicholas.
		* Dual MIT/BSD license
		* 
		*/
		window.matchMedia=window.matchMedia||(function(e,f){var c,a=e.documentElement,b=a.firstElementChild||a.firstChild,d=e.createElement("body"),g=e.createElement("div");g.id="mq-test-1";g.style.cssText="position:absolute;top:-100em";d.style.background="none";d.appendChild(g);return function(h){g.innerHTML='&shy;<style media="'+h+'"> #mq-test-1 { width: 42px; }</style>';a.insertBefore(d,b);c=g.offsetWidth==42;a.removeChild(d);return{matches:c,media:h}}})(document);
		
		// ---------------------------------------------------------------------------------------------------
		
	   /*
	    * NOTE: included here so as to avoid namespace clashes.
	    * Slightly modified to reference "window" at the end rather than "this"
	    * 
		* respond.js polyfill - enable media queries in IE 6-8
		* Author: Scott Jehl
		* Copyright (c) Scott Jehl.
		* Dual MIT/GPLv2 license
		* http://j.mp/respondjs
		* 
		*/
		(function(e){e.respond={};respond.update=function(){};respond.mediaQueriesSupported=e.matchMedia&&e.matchMedia("only all").matches;if(respond.mediaQueriesSupported){return}var w=e.document,s=w.documentElement,i=[],k=[],q=[],o={},h=30,f=w.getElementsByTagName("head")[0]||s,g=w.getElementsByTagName("base")[0],b=f.getElementsByTagName("link"),d=[],a=function(){var D=b,y=D.length,B=0,A,z,C,x;for(;B<y;B++){A=D[B],z=A.href,C=A.media,x=A.rel&&A.rel.toLowerCase()==="stylesheet";if(!!z&&x&&!o[z]){if(A.styleSheet&&A.styleSheet.rawCssText){m(A.styleSheet.rawCssText,z,C);o[z]=true}else{if((!/^([a-zA-Z:]*\/\/)/.test(z)&&!g)||z.replace(RegExp.$1,"").split("/")[0]===e.location.host){d.push({href:z,media:C})}}}}u()},u=function(){if(d.length){var x=d.shift();n(x.href,function(y){m(y,x.href,x.media);o[x.href]=true;u()})}},m=function(I,x,z){var G=I.match(/@media[^\{]+\{([^\{\}]*\{[^\}\{]*\})+/gi),J=G&&G.length||0,x=x.substring(0,x.lastIndexOf("/")),y=function(K){return K.replace(/(url\()['"]?([^\/\)'"][^:\)'"]+)['"]?(\))/g,"$1"+x+"$2$3")},A=!J&&z,D=0,C,E,F,B,H;if(x.length){x+="/"}if(A){J=1}for(;D<J;D++){C=0;if(A){E=z;k.push(y(I))}else{E=G[D].match(/@media *([^\{]+)\{([\S\s]+?)$/)&&RegExp.$1;k.push(RegExp.$2&&y(RegExp.$2))}B=E.split(",");H=B.length;for(;C<H;C++){F=B[C];i.push({media:F.split("(")[0].match(/(only\s+)?([a-zA-Z]+)\s?/)&&RegExp.$2||"all",rules:k.length-1,hasquery:F.indexOf("(")>-1,minw:F.match(/\(min\-width:[\s]*([\s]*[0-9\.]+)(px|em)[\s]*\)/)&&parseFloat(RegExp.$1)+(RegExp.$2||""),maxw:F.match(/\(max\-width:[\s]*([\s]*[0-9\.]+)(px|em)[\s]*\)/)&&parseFloat(RegExp.$1)+(RegExp.$2||"")})}}j()},l,r,v=function(){var z,A=w.createElement("div"),x=w.body,y=false;A.style.cssText="position:absolute;font-size:1em;width:1em";if(!x){x=y=w.createElement("body");x.style.background="none"}x.appendChild(A);s.insertBefore(x,s.firstChild);z=A.offsetWidth;if(y){s.removeChild(x)}else{x.removeChild(A)}z=p=parseFloat(z);return z},p,j=function(I){var x="clientWidth",B=s[x],H=w.compatMode==="CSS1Compat"&&B||w.body[x]||B,D={},G=b[b.length-1],z=(new Date()).getTime();if(I&&l&&z-l<h){clearTimeout(r);r=setTimeout(j,h);return}else{l=z}for(var E in i){var K=i[E],C=K.minw,J=K.maxw,A=C===null,L=J===null,y="em";if(!!C){C=parseFloat(C)*(C.indexOf(y)>-1?(p||v()):1)}if(!!J){J=parseFloat(J)*(J.indexOf(y)>-1?(p||v()):1)}if(!K.hasquery||(!A||!L)&&(A||H>=C)&&(L||H<=J)){if(!D[K.media]){D[K.media]=[]}D[K.media].push(k[K.rules])}}for(var E in q){if(q[E]&&q[E].parentNode===f){f.removeChild(q[E])}}for(var E in D){var M=w.createElement("style"),F=D[E].join("\n");M.type="text/css";M.media=E;f.insertBefore(M,G.nextSibling);if(M.styleSheet){M.styleSheet.cssText=F}else{M.appendChild(w.createTextNode(F))}q.push(M)}},n=function(x,z){var y=c();if(!y){return}y.open("GET",x,true);y.onreadystatechange=function(){if(y.readyState!=4||y.status!=200&&y.status!=304){return}z(y.responseText)};if(y.readyState==4){return}y.send(null)},c=(function(){var x=false;try{x=new XMLHttpRequest()}catch(y){x=new ActiveXObject("Microsoft.XMLHTTP")}return function(){return x}})();a();respond.update=a;function t(){j(true)}if(e.addEventListener){e.addEventListener("resize",t,false)}else{if(e.attachEvent){e.attachEvent("onresize",t)}}})(window);
		
		// ---------------------------------------------------------------------------------------------------
				
		function setDimensions() {
			// recalculate widths for responsive designs
			// get the current width and height of the panel
			if(responsive){
				wNew = getWidthFrom.width() * (percentWidth / 100);
				pNew = getWidthFrom.width() * (basePadding / getWidthFrom.width());	
			} else {
				wNew = panel.outerWidth();
			}
			
			hNew = panel.outerHeight(true);
			
			if(responsive){					
				panel.css({
					'width': wNew,
					'padding': pNew
				});
			} else {
				panel.css({
					'width': wNew
				});
			}
			
			float_container.css({
				//'width': wNew, // added for Paul's issue, might be a general issue though - needs testing!
				'min-height': hNew
			})
		}
		
		// ---------------------------------------------------------------------------------------------------
		
		function calculateUpperScrollBoundary() {
			// calculate the upper scrolling boundary			
			panelOffset = float_container.offset().top;
			panelMargin = parseFloat(float_container.css('marginTop').replace(/auto/, 0));
			realPanelOffset = panelOffset - panelMargin;
			topScrollBoundary = realPanelOffset - gap;
		}
					
		// ---------------------------------------------------------------------------------------------------	
			    
		// get the definitive options
		var opts = $.extend({}, $.fn.portamento.defaults, options);
		
		// setup the vars accordingly
		var panel = this;
		var wrapper = opts.wrapper;
		var gap = opts.gap;
		var disableWorkaround = opts.disableWorkaround;		
		var fullyCapableBrowser = positionFixedSupported();
		var responsive = opts.responsive;
		var percentWidth = opts.percentWidth;
		var getWidthFrom = opts.getWidthFrom;
		var minWidth = opts.minWidth;
		var basePadding = opts.basePadding;
		
		if(panel.length != 1) {
			// die gracefully if the user has tried to pass multiple elements 
			// (multiple element support is on the TODO list!) or no elements...
			return this;
		}

		if(!fullyCapableBrowser && disableWorkaround) {
			// just stop here, as the dev doesn't want to use the workaround
			return this;
		}

		// get the current width and height of the panel
		//w = panel.outerWidth();
		//h = panel.outerHeight();
				
		// wrap the panel in the container, then set the width and min-height accordingly
		panel.wrap('<div id="portamento_container" />');
		var float_container = $('#portamento_container');
				
		setDimensions();		
		calculateUpperScrollBoundary();
		// a couple of numbers to account for margins and padding on the relevant elements
		var wrapperPaddingFix = parseFloat(wrapper.css('paddingTop').replace(/auto/, 0));
		var containerMarginFix = parseFloat(float_container.css('marginTop').replace(/auto/, 0));
		// do some work to fix IE misreporting the document width
		var ieFix = 0;		
		var isMSIE = /*@cc_on!@*/0;		
		if (isMSIE) {
			ieFix = getScrollerWidth() + 4;
		}
						
		// ---------------------------------------------------------------------------------------------------
		
		function scrollHandler() {
						
			if(thisWindow.height() > parseInt(panel.outerHeight(true) + gap) && thisWindow.width() >= (thisDocument.width() - ieFix)  && (matchMedia('only screen and (min-width: '+minWidth+'px)').matches)) { // don't scroll if the window isn't big enough
				//(!responsive || (thisWindow.width() + getScrollerWidth()) > minWidth)
				var y = thisDocument.scrollTop(); // current scroll position of the document
												
				if (y >= (topScrollBoundary)) { // if we're at or past the upper scrolling boundary
					if((panel.innerHeight() - wrapper.viewportOffset().top) - wrapperPaddingFix + gap >= wrapper.height()) { // if we're at or past the bottom scrolling boundary
						if(panel.hasClass('fixed') || thisWindow.height() >= panel.outerHeight()) { // check that there's work to do
							panel.removeClass('fixed');
							panel.css('top', (wrapper.height() - panel.innerHeight()) + 'px');
						}
					} else { // if we're somewhere in the middle
						panel.addClass('fixed');
						
						if(fullyCapableBrowser) { // supports position:fixed
							panel.css('top', gap + 'px'); // to keep the gap
						} else {							
							panel.clearQueue();
							panel.css('position', 'absolute').animate({top: (0 - float_container.viewportOffset().top + gap)});
						}
					}
				} else {
					// if we're above the top scroll boundary
					panel.removeClass('fixed');
					panel.css('top', '0'); // remove any added gap
				}
			} else {
				panel.removeClass('fixed');
			}
		}
		
		thisWindow.bind("scroll.portamento", scrollHandler);
		
		// ---------------------------------------------------------------------------------------------------
		
		thisWindow.bind("resize.portamento", function () {
											
			// make sure Portamento keeps in sync with the actual element
			if(responsive)
			{				
				setDimensions();
				calculateUpperScrollBoundary();
			}	
			
			// stop users getting undesirable behaviour if they resize the window too small
			if(thisWindow.height() <= panel.outerHeight() || thisWindow.width() < thisDocument.width() || (responsive && matchMedia('only screen and (max-width: '+minWidth+'px)').matches)) {					
				if(panel.hasClass('fixed')) {
					panel.removeClass('fixed');
					panel.css('top', '0');
				}				
			} else {
				scrollHandler(); // trigger the scroll event to place the panel correctly
			}		
		});
		
		// ---------------------------------------------------------------------------------------------------
		
		thisWindow.bind("orientationchange.portamento", function () {
			// if device orientation changes, trigger the resize event
			thisWindow.trigger('resize.portamento'); 
		});
		
		// ---------------------------------------------------------------------------------------------------
		
		thisWindow.load(function() {
			calculateUpperScrollBoundary(); // calculate the upper scroll boundary when everything has loaded to avoid reflow issues
			setInterval(function(){ calculateUpperScrollBoundary(); }, 500); // and every 500ms to cover anything else that gets loaded
		});
		
		// trigger the scroll event immediately so that the panel is positioned correctly if the page loads anywhere other than the top.
		thisWindow.trigger('scroll.portamento');
		
	    // return this to maintain chainability
	    return this;	
	};
	
	// set some sensible defaults
	$.fn.portamento.defaults = {
	  'wrapper'				: $('body'),// the element that will act as the sliding panel's boundaries
	  'gap'					: 10, 		// the gap (in pixels) left between the top of the viewport and the top of the panel
	  'disableWorkaround'	: false, 	// option to disable the workaround for not-quite capable browsers
	  'responsive'			: false, 	// toggles responsive mode on or off
	  'percentWidth'		: 100, 		// the width (in percent) of the panel relative to its parent (responsive mode only)
	  'getWidthFrom'		: $('body'),// the element that the panel takes its width from (responsive mode only)
	  'minWidth'			: 0, 		// the minimum width (in pixels) at which the panel will slide (responsive mode only)
	  'basePadding'			: 0			// the base padding for the sliding element, in pixels (responsive mode only)
	};
	
})(jQuery);